unit CoolButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;
type
  TCoolButton = class(TGraphicControl)
  private
    { Private declarations }
    FImageUp: TBitmap;
    FImageDown: TBitmap;
    FCaption: String;
    isPressed: Boolean;
    procedure SetImageUp(bmp: TBitmap);
    procedure SetImageDown(bmp: TBitmap);
    procedure SetCaption(strcaption: String);
  protected
    { Protected declarations }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure Paint(); override;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property ImageUp: TBitmap
        read FImageUp write SetImageUp;
    property ImageDown: TBitmap
        read FImageDown write SetImageDown;
    property Caption: String
        read FCaption write SetCaption;
    property ShowHint;    
    property Font;
    property OnClick;
    property DragCursor;        { drag-and-drop properties }
    property DragMode;
    property OnDragDrop;        { drag-and-drop events }
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;       { mouse events }
    property OnMouseMove;
    property OnMouseUp;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('ARIOX.COM', [TCoolButton]);
end;

constructor TCoolButton.Create(AOwner:TComponent);
begin
    inherited;
    FImageUp := TBitmap.Create();
    FImageDown := TBitmap.Create();
    self.Height := 76;
    self.Width := 76;
    isPressed := false;
    Invalidate();
end;

destructor TCoolButton.Destroy;
begin
    FImageUp.Free;
    FImageDown.Free;
    inherited Destroy;
end;
procedure TCoolButton.SetImageDown(bmp: TBitmap);
begin
    FImageDown.Assign(bmp);
    Invalidate();
end;

procedure TCoolButton.SetImageUp(bmp: TBitmap);
begin
    FImageUp.Assign(bmp);
    Invalidate();
end;

procedure TCoolButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
     if Button = mbLeft then
     begin
        isPressed := true;
        Invalidate;

     end;
end;

procedure TCoolButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
     if Button = mbLeft then
     begin
            isPressed := false;
            Invalidate;
     end;

end;



procedure TCoolButton.Paint();
var
rect: TRect;
h,w,x,y, textWidth, textHeight: Integer;
begin
if isPressed then
begin
    if FImageDown.Height > 0 then begin
      self.Width := FImageDown.Width;
      self.Height := FImageDown.Height;
      rect.Top := 0;
      rect.Left := 0;
      rect.Bottom := FImageDown.Height;
      rect.Right := FImageDown.Width;
      self.Canvas.Brush.Style := bsClear;
      self.Canvas.Pen.Style := psDashDot;
      self.Canvas.FillRect(rect);
      self.Canvas.Draw(0,0, FImageDown);
    end else begin
      self.Width := 76;
      self.Height := 76;
      rect.Top := 0;
      rect.Left := 0;
      rect.Bottom := 76;
      rect.Right := 76;
      if csDesigning in ComponentState then begin
        self.Canvas.Brush.Style := bsClear;
        self.Canvas.Pen.Style := psDashDot;
        self.Canvas.Rectangle(rect);
      end;
    end;
end else
begin
    if FImageUp.Height > 0 then begin
      self.Width := FImageUp.Width;
      self.Height := FImageUp.Height;
      rect.Top := 0;
      rect.Left := 0;
      rect.Bottom := FImageUp.Height;
      rect.Right := FImageUp.Width;
      self.Canvas.Brush.Style := bsClear;
      self.Canvas.Pen.Style := psDashDot;
      self.Canvas.FillRect(rect);
      self.Canvas.Draw(0,0, FImageUp);

    end else begin
      self.Width := 76;
      self.Height := 76;
      rect.Top := 0;
      rect.Left := 0;
      rect.Bottom := 76;
      rect.Right := 76;
      if csDesigning in ComponentState then begin
      self.Canvas.Brush.Style := bsClear;
      self.Canvas.Pen.Style := psDashDot;
      self.Canvas.Rectangle(rect);
      end;
    end;
end;

if FCaption <> '' then begin
  h := self.Height;
  w := self.Width;
  self.Canvas.Font.Assign(Font);
  textWidth := self.Canvas.TextWidth(FCaption);
  textHeight := self.Canvas.TextHeight(FCaption);
  x := (w - textWidth) div 2;
  y := (h - textHeight) div 2;
  rect.Left := 0;
  rect.Top := 0;
  rect.Right :=  w;
  rect.Bottom := h;
  self.Canvas.TextOut(x,y,FCaption);
end;
end;

procedure TCoolButton.SetCaption(strcaption: String);
begin
FCaption := strCaption;
Invalidate();
end;

end.
