unit NetworkPrinter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Printers;

type
  TNetworkPrinter = class(TComponent)
  private
    { Private declarations }
    charHeight, charWidth, maxLinesPerPage, maxCharsPerLine: Integer;
    FFont: TFont;
    FTitle: String;
    FOnJobCompleted: TNotifyEvent;
    FPrinterOrientation: TPrinterOrientation;
    procedure SetTitle(title: String);
    function GetCurrentPage: Integer;
  protected
    { Protected declarations }
    procedure DoOnJobCompleted; virtual;
  public
    procedure WriteData(column, line: Integer; text: String);
    procedure SetSettings;
    procedure NewPage;
    procedure BeginDoc;
    procedure EndDoc;
    procedure DrawImage(var rect: TRect; var bmp: TBitmap);
    function SelectPrinter: Boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBoldFont;
    procedure SetRegularFont;
    { Public declarations }

  published
    { Published declarations }
    property Font: TFont
        read FFont write FFont;
    property Title: String
        read FTitle write SetTitle;
    property CurrentPage: Integer
        read GetCurrentPage;
    property PixelCharWidth: Integer
        read charWidth;
    property PixelCharHeight: Integer
        read charHeight;
    property LinesPerPage: Integer
        read maxLinesPerPage;
    property CharsPerLine: Integer
        read maxCharsPerLine;
    property OnJobCompleted: TNotifyEvent
        read FOnJobCompleted write FOnJobCompleted;
    property Orientation: TPrinterOrientation
        read FPrinterOrientation write FPrinterOrientation;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('ARIOX.COM', [TNetworkPrinter]);
end;

constructor TNetworkPrinter.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    FFont := TFont.Create();
    FTitle := 'My Report';
    FFont.Name := 'Courier New';
    FFont.Size := 10;
    FFont.Style := [];
    self.Font.Assign(FFont);
end;

destructor TNetworkPrinter.Destroy;
begin
   FFont.Free();
end;
procedure TNetworkPrinter.BeginDoc;
begin
    Printer.Orientation := FPrinterOrientation;
    Printer.BeginDoc;
end;

procedure TNetworkPrinter.DrawImage(var rect: TRect; var bmp: TBitmap);
begin
  Printer.Canvas.StretchDraw(rect, bmp);
end;

procedure TNetworkPrinter.EndDoc;
begin
    Printer.EndDoc;
    DoOnJobCompleted;
end;

procedure TNetworkPrinter.DoOnJobCompleted;
begin
   if Assigned(FOnJobCompleted) then
    FOnJobCompleted(self);
end;
function TNetworkPrinter.GetCurrentPage: Integer;
begin
  Result:= Printer.PageNumber;
end;

procedure TNetworkPrinter.NewPage;
begin
  Printer.NewPage;
end;

function TNetworkPrinter.SelectPrinter: Boolean;
var
    printDialog: TPrintDialog;
    wasSelected: Boolean;
begin
    printDialog := TPrintDialog.Create(self);
    if printDialog.Execute then
       wasSelected := true
    else
       wasSelected := false;
    printDialog.Free();
    Result := wasSelected;
end;

procedure TNetworkPrinter.SetSettings;
begin
  Printer.Canvas.Font := FFont;
  charWidth := Printer.Canvas.TextWidth('W');
  charHeight:= Printer.Canvas.TextHeight('W');
  maxLinesPerPage := Printer.PageHeight div charHeight;
  maxCharsPerLine := Printer.PageWidth div charWidth;
end;

procedure TNetworkPrinter.SetTitle(title: String);
begin
  Printer.Title := title;
  FTitle := title;
end;

procedure TNetworkPrinter.WriteData(column, line: Integer; text: String);
begin
     Printer.Canvas.TextOut (column * charWidth, line * charHeight, text);
end;


procedure TNetworkPrinter.SetBoldFont;
begin
    Printer.Canvas.Font.Style := [fsBold];
end;

procedure TNetworkPrinter.SetRegularFont;
begin
    Printer.Canvas.Font.Style := [];
end;

end.
