unit SortedListView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, DsgnIntf;

type
  TSortedListView = class(TListView)
  private
    { Private declarations }
    ColumnToSort: Integer;
    sortAscending: Boolean;
    ImageList: TImageList;
    idx1,idx2: Integer;
    lastColumn: Integer;
    FDownArrow: TIcon;
    FUpArrow: TIcon;
    procedure DoCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    procedure SetDownArrow(icon: TIcon);
    procedure SetUpArrow(icon: TIcon);
  protected
    { Protected declarations }
    procedure ColClick(Column: TListColumn); override;

  public
    { Public declarations }
    procedure SetImages;

  published
        property DownArrow: TIcon
            read FDownArrow write SetDownArrow;
        property UpArrow: TIcon
            read FUpArrow write SetUpArrow;
        constructor Create(AOwner: TComponent); override;
        destructor Destroy; override;

  end;

procedure Register;

implementation


procedure Register;
begin
  RegisterComponents('ARIOX.COM', [TSortedListView]);
end;

constructor TSortedListView.Create(AOwner: TComponent);     { this goes after implementation }
begin
  inherited Create(AOwner);                           { ALWAYS do this first! }
  FDownArrow := TIcon.Create();
  FUpArrow := TIcon.Create();
  ViewStyle := vsReport;
  OnCompare := DoCompare;                                  { set the new desired value }
  self.SortType := stData;
  lastColumn := -1;
  self.sortAscending := false;
end;

destructor TSortedListView.Destroy;

begin
  if Assigned(ImageList) then
      ImageList.Free();
  FDownArrow.Free();
  FUpArrow.Free();
  inherited Destroy;
end;
procedure TSortedListView.DoCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
var
  ix, direction: Integer;
begin
  if ColumnToSort = 0 then begin
    Compare := CompareText(Item1.Caption,Item2.Caption);
    if not sortAscending then begin
        Compare := Compare * -1;
    end;
  end
  else begin
   ix := ColumnToSort - 1;
   if not sortAscending then
        direction := -1
   else
        direction := 1;
   Compare := CompareText(Item1.SubItems[ix],Item2.SubItems[ix]) * direction;
  end;
end;

procedure TSortedListView.ColClick(Column: TListColumn);
var
    col: TListColumn;
begin
if lastColumn <> -1 then begin
   col := self.Columns.Items[lastColumn];
   col.ImageIndex := -1;
end;
sortAscending := not sortAscending;
ColumnToSort := Column.Index;
if lastColumn <> ColumnToSort then
    sortAscending := true;
(self as TCustomListView).AlphaSort;


col :=  self.Columns.Items[ColumnToSort];
if not sortAscending then
      col.ImageIndex := idx2
else
      col.ImageIndex := idx1;

lastColumn := ColumnToSort;
end;

procedure TSortedListView.SetImages;
begin
   ImageList := TImageList.Create(self);
   idx1 := ImageList.AddIcon(FDownArrow);
   idx2 := ImageList.AddIcon(FUpArrow);
   self.SmallImages := ImageList;
end;

procedure TSortedListView.SetDownArrow(icon: TIcon);
begin
    FDownArrow.Assign(icon);
end;

procedure TSortedListView.SetUpArrow(icon: TIcon);
begin
    FUpArrow.Assign(icon);
end;



end.
