unit WebLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ShellApi, mmsystem, DsgnIntf, SiteForm;

type
  TErrorLoading =
        procedure(Sender: TObject; ErrorMessage: String) of object;
  TWebLabel = class(TCustomLabel)
  private
    { Private declarations }
    FSite: String;
    FColor : TColor;
    FClickColor: TColor;
    FHoverColor: TColor;
    FEnableHover: Boolean;
    FOnErrorLoading: TErrorLoading;
    FOnHover: TNotifyEvent;
  protected
    { Protected declarations }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
    procedure MouseEnter(var Msg: TMessage); message cm_mouseEnter;
    procedure MouseLeave(var Msg: TMessage); message cm_mouseLeave;
    procedure DoErrorLoading; virtual;
    procedure DoHoverAction; virtual;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent);override;
  published
    { Published declarations }
    property Site: String
        read FSite write FSite;
    property ClickColor: TColor
        read FClickColor write FClickColor default clRed;
    property EnableHover: Boolean
        read FEnableHover write FEnableHover default True;
    property HoverColor: TColor
        read FHoverColor write FHoverColor default clTeal;
    property Align;
    property Alignment;
    property AutoSize;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property Cursor;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint default True;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;

    property OnErrorLoading: TErrorLoading
        read FOnErrorLoading write FOnErrorLoading;
    property OnHover: TNotifyEvent
        read FOnHover write FOnHover;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

  TSiteProperty = class (TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure Edit; override;
  end;

  TWebLabelCompEditor = class (TComponentEditor)
    function GetVerbCount: Integer; override;
    function GetVerb(Index: Integer): string; override;
    procedure ExecuteVerb(Index: Integer); override;
    procedure Edit; override;
  end;
procedure Register;

implementation

procedure Register;
begin
  RegisterComponentEditor(TWebLabel, TWebLabelCompEditor);  
  RegisterPropertyEditor(TypeInfo(string),TWebLabel, 'Site', TSiteProperty);
  RegisterComponents('ARIOX.COM', [TWebLabel]);
  RegisterPropertyInCategory(TVisualCategory, TWebLabel, 'EnableHover');
end;

constructor TWebLabel.Create(AOwner: TComponent);     { this goes after implementation }
begin
  inherited Create(AOwner);                           { ALWAYS do this first! }
  Transparent := true;
  Cursor := crHandPoint;
  FClickColor := clRed;
  FHoverColor := clTeal;
  FEnableHover := true;
  self.Font.Color := clBlue;
  ShowHint := true;
  Hint := 'Visit our web site';

end;

procedure TWebLabel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FColor := self.Font.Color;
  if Button = mbLeft then
          self.Font.Color := FClickColor;
end;

procedure TWebLabel.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
        returnedValue: Longint;
begin
  if Button = mbLeft then begin
        self.Font.Color := FColor;
        returnedValue := ShellExecute(0,'open', PChar(self.Site), nil, nil, SW_SHOWNORMAL);
        if returnedValue <= 32 then
                DoErrorLoading;
  end;
end;


procedure TWebLabel.DoErrorLoading;
begin
    if Assigned(FOnErrorLoading) then
        FOnErrorLoading(self, 'Unable to load browser');
end;

procedure TWebLabel.DoHoverAction;
begin
    if Assigned(FOnHover) then
        FOnHover(self);
end;

procedure TWebLabel.MouseEnter(var Msg: TMessage);
begin
        if FEnableHover then
        begin
                self.Font.Color := FHoverColor;
                DoHoverAction;
        end;
end;

procedure TWebLabel.MouseLeave(var Msg: TMessage);
begin
        if FEnableHover then
        begin
                self.Font.Color := clBlue;
        end;
end;

function TSiteProperty.GetAttributes: TPropertyAttributes;
begin
        Result := [paDialog];
end;

procedure TSiteProperty.GetValues(Proc: TGetStrProc);
begin
    //
end;

procedure TSiteProperty.Edit;
var
   sf: TfrmSiteForm;
begin
   sf := TfrmSiteForm.Create(Application);
   try
    sf.txtSite.Text := GetValue;
    if sf.ShowModal = mrYes then
        SetValue(sf.txtSite.Text);
    finally
    sf.Free;
   end;
end;

    function TWebLabelCompEditor.GetVerbCount: Integer;
    begin
       Result := 1;
    end;
    function TWebLabelCompEditor.GetVerb(Index: Integer): string;
    begin
        case Index of
            0: Result := 'About Alexis Ros';
        end;
    end;
    procedure TWebLabelCompEditor.ExecuteVerb(Index: Integer);
    begin
        case Index of
            0: MessageBox(0, PChar('TWebLabel - Alexis Ros Components II' + #10 + #10 + 'Copyright 2000 Alexis Ros Negrn'), 'About TWebLabel', MB_ICONINFORMATION);
        end;
    end;
    procedure TWebLabelCompEditor.Edit;
    begin
        Beep;
        ExecuteVerb(0);
    end;

end.

